
/* print.c */

#include <math.h>
#include <malloc.h>
#include <string.h>
#include "types.h"
#include "proto.h"
#include "select.h"
#include "matrix.h"
#include "print.h"

/* indices for display[] */
enum disp_index {WIDTH = 0, HEIGHT = 1};

/* local prototypes */
static bool inc_linecount(void);
static int PrintCentredString(FILE * where, const string s,
           const ScreenSize fieldwidth, const string after);
static char symbol(const elem_t id);
static char * nextpos(char curr[], char prev[], const elem_t maxlen);
static char nextsymbol(const elem_t group, const bool firsttime);

/* module global variables */
static const char       linechar     = '-';      
static       ScreenSize lines        = 0;        /* count of lines used */
static       ScreenSize display[]    = {80, 24}; /* width & height */
static       bool       scroll_menus = FALSE;    

/*                                                                         *\
   -----------------------------------------------------------------------
                         Display height and width
   -----------------------------------------------------------------------
\*                                                                         */

ScreenSize height_display(void)
{
   return (display[HEIGHT]);
} /* height_display() */


void set_height(const ScreenSize new_size)
{
   display[HEIGHT] = new_size;
} /* set_height() */


ScreenSize width_display(void)
{
   return (display[WIDTH]);
} /* width_display() */


void set_width(const ScreenSize new_size)
{
   display[WIDTH] = new_size;
} /* set_width() */


/*                                                                         *\
   -----------------------------------------------------------------------
                            Screen Height Code
   -----------------------------------------------------------------------
\*                                                                         */

void init_print_count(void) 
{
   lines = 0;
} /* init_print_count() */


bool inc_linecount(void) 
{
   bool status;

   if ((0 != display[HEIGHT]) && (++lines >= (display[HEIGHT] - 1))) {
         status = wait_for_enter_key();
         init_print_count();
         return(status);
   } else {
      return (TRUE);
   }
} /* inc_linecount() */


bool end_display(void) 
{
   bool status;

   if ((0 != display[HEIGHT]) && (0 != lines)) {
         status = wait_for_enter_key();
         init_print_count();
         return(status);
   } else {
      return (TRUE);
   }
} /* end_display() */

/*                                                                         *\
   -----------------------------------------------------------------------
                 Scroll Menus (to start at top of display)
   -----------------------------------------------------------------------
\*                                                                         */

bool s_m_value(void) 
{
   return(scroll_menus);
} /* s_m_value() */


void s_m_toggle(void)
{
   scroll_menus = (! scroll_menus);
} /* s_m_toggle() */


bool start_menu(void)
{
   register ScreenSize i;
   const    ScreenSize height = height_display();
   register bool       status = TRUE;

   if (scroll_menus) {
      for (i=0; status && (i < height); i++) {
         status = (EOF != putchar('\n'));
      }
      return(status);
   } else {
      return (TRUE);
   }
} /* start_menu() */


bool end_menu(void)
{
/*
   end_menu():
     PURPOSE: Forces the program to pause (until Return is pressed) when
              the display is about tho be cleared at the start of a menu.
     PRE:     In a menu just after informative message, e.g. `Cannot delete
              participants since none are selected', has been printed.
     POST:    Returns FALSE iff an I/O error was detected */

   return (s_m_value()) ?wait_for_enter_key() :TRUE;
} /* end_menu() */

/*                                                                         *\
   -----------------------------------------------------------------------
                               Print Tables
   -----------------------------------------------------------------------
\*                                                                         */

elem_t s_print(const chose_t *  select,
               const bool       print_all,
               const bool       val_to_print,
                     elem_t     printed[],
               const ScreenSize displaywidth) {

   register elem_t   i;        /* loop index through all names */
   register elem_t   count;    /* count of number printed */
   register elem_t   thisline; /* count of number printed on current line */
   register int      status = TRUE;
            elem_t   max_count;
            unsigned across = (int)floor((double)displaywidth /        \
                                         (double)(select->max_len + 6));

   if (0 == across) { across = 1; }
   if (! print_all) {
      max_count = (val_to_print) ?s_count(select)               \
                                 :select->size - s_count(select);
   } else {
      max_count = select->size;
   }

   for (count=i=thisline=0; count < max_count; i++){
      if (print_all || (s_value(select,i) == val_to_print)) {
         thisline++;
         printed[count++] = i;
         status=printf("%3lu. %-*s%c",                               \
                       count, (int)select->max_len, select->name[i], \
                       (thisline == across)?'\n':' ');
         if (thisline == across) { 
            thisline = 0;
            inc_linecount();
         }
      }
   } /* for */
   status |= puts("");
   if (EOF != status) {
      status |= (inc_linecount())?0:EOF;
   }
   return( (EOF != status)?count:0 );
} /* s_print() */


void print_raw_data(const unsigned   maxpartiwidth,
                    const ScreenSize maxquerywidth,
                    const Matrix *   data,
                    const string     parti_name[],
                    const string     query_name[],
                    const string     subtitle,
                    const bool       reset_line_count) 
{
                   char       proc[] = "print_raw_data";
   extern const    char       linechar;
          register matrix_dim query, parti, count;
          register matrix_dim numtoprint = num_rows(data);
                   matrix_dim particount = num_cols(data);
          register unsigned   startat = 0; 
                   ScreenSize fieldsize = \
                       ((maxquerywidth > REAL_WIDTH)?maxquerywidth:REAL_WIDTH);
                   ScreenSize across    = \
                       (width_display() - maxpartiwidth - 1) / (fieldsize + 1);
                   ScreenSize groupsize; /* # of query vals to print across */
                   string     title, 
                              level;

   start_menu();
   if (0 == numtoprint) {
      swerr(proc, "No queries selected", 0);
   }
   if (0 == particount) {
      swerr(proc, "No participants selected", 0);
   }
   if (!particount || !numtoprint) {
      return;
   }

   if (reset_line_count) {
      init_print_count();
   }
   PrintLine(stdout, linechar, width_display(), "\n");
   inc_linecount();
   if (NULL != (title = mat_title(data))) {
      PrintCentredString(stdout, title, width_display(), "\n");
      inc_linecount();
   }
   if (NULL != (level = mat_level(data))) {
      PrintCentredString(stdout, level, width_display(), "\n");
      inc_linecount();
   }
   PrintCentredString(stdout, subtitle, width_display(), "\n");
   inc_linecount();
   PrintLine(stdout, linechar, width_display(), "\n");
   inc_linecount();

   while (numtoprint > 0) {
      groupsize = (numtoprint < across) ?numtoprint :across;

      printf("%*s", (int)maxpartiwidth, "Participant");
      PrintCentredString(stdout, "Query", ((fieldsize + 1) * groupsize), "\n");
      inc_linecount();

      /* query names at top of columns */
      printf(" %*s", (int)maxpartiwidth, " ");
      for (query=startat,count=0; count < groupsize; query++) {
         printf("%-*s %c", (int)fieldsize - 1, query_name[query], \
                           (count == groupsize - 1)?'\n' :' ');   
         count++;
      }
      inc_linecount();
      if (groupsize > count) {
         NewLine(1, stdout);
      }

      /* print parti names and data */
      for (parti=0; parti < particount; parti++){
         printf("%*s ", (int)maxpartiwidth, parti_name[parti]);
         for(query=startat,count=0; count < groupsize; query++) {
            count++;
            printf(REAL_SPEC"%c", mat_value(data,query,parti), \
                (0 == (count % groupsize)) ?'\n' :' ');
         } /* for i = 0..groupsize */
         inc_linecount();
      } /* for parti */
      NewLine(1, stdout);

      numtoprint -= groupsize;
      startat += groupsize;
   }
} /* print_raw_data() */



void print_ave_data(const real       average[],
                    const matrix_dim howmany,
                    const ScreenSize maxwidth, /* how wide can name be? */
                    const string     name[],   /* queries or participants */
                    const bool       byparti,
                    const string     levels,
                    const string     title,
                    const string     subtitle,
                    const bool       reset_line_count) 
{
   #define MAX_WIDTH (80)
   extern   const    char       linechar;
                     char       average_type[MAX_WIDTH + 1];
   register          matrix_dim i;
                     ScreenSize width = width_display();
   
   start_menu();
   if (reset_line_count) {
      init_print_count();
   }
   strcpy(average_type, "Average by ");
   strcat(average_type, (byparti)?"participant":"query");

   PrintLine(stdout, linechar, width, "\n");
   inc_linecount();
   PrintCentredString(stdout, title, width, "\n");
   inc_linecount();
   if (NULL != subtitle) {
      PrintCentredString(stdout, subtitle, width, "\n");
      inc_linecount();
   }
   if (NULL != levels) {
      PrintCentredString(stdout, levels, width, "\n");
      inc_linecount();
   }
   PrintCentredString(stdout, average_type, width, "\n");
   inc_linecount();
   PrintLine(stdout, linechar, width, "\n");
   inc_linecount();

   printf("%*s Average\n", (int)maxwidth, (byparti)?"Participant" :"Query");
   inc_linecount();
   for (i=0; i < howmany; i++) {
       printf("%*s "REAL_SPEC"\n", (int)maxwidth, name[i], average[i]);
       inc_linecount();
   }
   #undef MAX_WIDTH
} /* print_ave_data() */



void print_both_ave(const string levels,
                    const string title,
                    const string subtitle,
                    const real   average,
                    const bool   reset_line_count) 
{
   extern   const char       linechar;
                  char       average_type[] = \
                               "Average by participant and query";
   register const ScreenSize width = width_display();

   start_menu();
   if (reset_line_count) {
      init_print_count();
   }

   PrintLine(stdout, linechar, width, "\n");
   inc_linecount();
   PrintCentredString(stdout, average_type, width, "\n");
   inc_linecount();
   PrintCentredString(stdout, title, width, "\n");
   inc_linecount();
   if (NULL != levels) {
      PrintCentredString(stdout, levels, width, "\n");
      inc_linecount();
   }
   if (NULL != subtitle) {
      PrintCentredString(stdout, subtitle, width, "\n");
      inc_linecount();
   }

   PrintLine(stdout, linechar, width, "\n");
   inc_linecount();

   printf(REAL_SPEC"\n", average);
   inc_linecount();
} /* print_both_ave() */


/*                                                                         *\
   -----------------------------------------------------------------------
                               ANOVA Table
   -----------------------------------------------------------------------
\*                                                                         */

void ANOVATable(      ATableEntry  entry[],
                const unsigned int size,
                const string       title,
                const string       datatype,
                const string       level_str,
                const string       note,
                      FILE *       outfile)
{
   #define F_W /* field_width */ 14
   static   const ScreenSize   ANOVA_table_width = 80;
   register       unsigned int i;

   PrintLine(outfile, linechar, ANOVA_table_width, "\n");
   inc_linecount();
   if (title) {
      PrintCentredString(outfile, title, ANOVA_table_width, "\n");
      inc_linecount();
   }
   if (datatype) {
      PrintCentredString(outfile, datatype, ANOVA_table_width, "\n");
      inc_linecount();
   }
   if (level_str) {
      PrintCentredString(outfile, level_str, ANOVA_table_width, "\n");
      inc_linecount();
   }
   if (note) {
      PrintCentredString(outfile, note, ANOVA_table_width, "\n");
      inc_linecount();
   }
   PrintLine(outfile, linechar, ANOVA_table_width, "\n");
   inc_linecount();

   fprintf (outfile, "%-*s  %-11s  %-*s  %-*s  %-9s  %-6s\n",        \
                     F_W, "Source of", "Degrees of", F_W, "Sum  of", \
                     F_W, " Mean", "F", "Critical");
   inc_linecount();
   fprintf (outfile, "%-*s  %-11s  %-*s  %-*s  %-9s  %-6s\n",     \
                     F_W, "Variation", "Freedom", F_W, "Squares", \
                     F_W, "Square", "", "Value");
   inc_linecount();
   PrintLine(outfile, linechar, ANOVA_table_width, "\n");
   inc_linecount();

   for (i=0; i < size; i++) {
      const string seperator = " -,";
            string word, 
                   descrip   = strdup(entry[i].srcvar),
                   toprint   = (char *)malloc(strlen(entry[i].srcvar));
            int    spaceleft;

       if ((NULL == descrip) || (NULL == toprint)) {
          if (NULL != descrip) {
             free(descrip);
	  }
          fprintf(outfile, "%-s\n%-*s", entry[i].srcvar, F_W, "");
       } else {
          toprint[0] = '\0';
          spaceleft = F_W;
          word = strtok(descrip, seperator);
          while (word) {
             if (strlen(word) <= (size_t)spaceleft) {
                strcat(toprint, word);
                spaceleft -= strlen(word);
                if (spaceleft > 0) {
                   strcat(toprint, " ");
                   spaceleft--;
		}
                word = strtok(NULL, seperator);
	     } else {
                if (F_W == spaceleft) {
                   fprintf(outfile, "%s", word);
                   NewLine(1, outfile);
                   word = strtok(NULL, seperator);
		} else {
                   fprintf(outfile, "%-*s", F_W, toprint);
                   spaceleft = F_W;
                   toprint[0] = '\0';
                   if (NULL != word) {
                      NewLine(1, outfile);
		   }
		}
	     }
	  } /* while */
          fprintf(outfile, "%-*s", F_W, toprint);
       }
       fprintf(outfile, " %6lu %6s %-*g  ", \
              entry[i].degfreed, "", F_W, entry[i].sumsquare);
       if ((size - 1) != i) {
          fprintf(outfile, "%-*g", F_W, entry[i].meansquare);
       }
       if ((size - 2) > i) {
          fprintf(outfile, " %-9.4f %c %-6.4f",           \
                           entry[i].f_value,              \
                           (entry[i].signifdiff)?'*':' ', \
                           entry[i].critval);
       }
       NewLine(2, outfile);
   }
   PrintLine(outfile, linechar, ANOVA_table_width, "");
   NewLine(2, outfile);
   #undef F_W
} /* ANOVATable() */


/*                                                                         *\
   -----------------------------------------------------------------------
                       Miscellaneous output routines
   -----------------------------------------------------------------------
\*                                                                         */

int NewLine(unsigned howmany, 
            FILE *   outfile) 
{
   int status = 0;

   for (;howmany > 0; howmany--) {
      status |= fputc('\n', outfile);
      inc_linecount();
   }
   return(status);
} /* NewLine() */


int PrintLine(      FILE *     where,
              const char       what,
              const ScreenSize fieldwidth,
              const string     after) 
{
   register int        status = 0;
   register ScreenSize i;

   for (i=0; i < fieldwidth; i++) {
      status |= fputc(what, where);
   }
   status |= fputs(after, where);
   return(status);
} /* PrintLine() */


int PrintCentredString(      FILE *     where,
                       const string     s, 
                       const ScreenSize fieldwidth,
                       const string     after)
{
   int pad = (fieldwidth - strlen(s)) / 2;

   if (pad > 0) {
      return(fprintf(where, "%*s%s%*s", pad, " ", s, pad, " ") | \
             fputs(after, where)); 
   } else {
      return(fprintf(where, "%s%s", s, after)); 
   }
} /* PrintCentredString() */


bool PrintTitle(      FILE *     outfile,
                const string     title, 
                const ScreenSize width)
{
   bool status = TRUE;

   status = (EOF != PrintLine(outfile, linechar, width, "\n"));
   status = status && inc_linecount();
   status = status && (EOF != PrintCentredString(outfile, title, width, "\n"));
   status = status && inc_linecount();
   status = status && (EOF != PrintLine(outfile, linechar, width, "\n"));
   status = status && inc_linecount();

   return(status);
} /* PrintTitle() */


/*                                                                         *\
   -----------------------------------------------------------------------
                                Mean Groups
   -----------------------------------------------------------------------
\*                                                                         */

int show_means(const Means      average[],
               const matrix_dim howmany,
               const string     title,    /* e.g. "Average precision" */
               const string     subtitle, /* e.g. "transformed" */
               const string     meantype, /* e.g. "participant" */
               const string     levels,
               const bool       showall,
                     FILE *     outfile)
{
   const    char       proc[] = "show_means";
            ScreenSize screenwidth = width_display();
            char       titlestr[MAXscreenwidth];
   register matrix_dim parti;
            elem_t     maxgroup,
                       group;
            char *     pos;
            char       dispchar;
            char       display[MAX_PARTICIPANT + 1];
            char       previous[MAX_PARTICIPANT + 1];

   PrintLine(outfile, linechar, screenwidth, "\n"); 
   inc_linecount();
   sprintf(titlestr, "Tukey T test grouping of %s%smeans", \
                     meantype, (meantype)?" ":"");
   PrintCentredString(outfile, titlestr, screenwidth, "\n");
   inc_linecount();
   PrintCentredString(outfile, title, screenwidth, "\n");
   inc_linecount();
   if (NULL != levels) {   
      PrintCentredString(outfile, levels, screenwidth, "\n");
      inc_linecount();
   }
   if (NULL != subtitle) {   
      PrintCentredString(outfile, subtitle, screenwidth, "\n");
      inc_linecount();
   }
   PrintLine(outfile, linechar, screenwidth, "\n");
   inc_linecount();
/*
   if (meantype) {
      fprintf(outfile, "%-*s %-*s",                         \
              REAL_WIDTH, "mean", MAX_PARTICIPANT, meantype);
      NewLine(1, outfile);
   }
*/

   sprintf(display,  "%*s", (int)MAX_PARTICIPANT, " ");
   sprintf(previous, "%*s", (int)MAX_PARTICIPANT, " ");
   for (parti=0; parti < howmany; parti++) {
      fprintf(outfile, REAL_SPEC" %-*s ",         \
             average[parti].mean, PARTI_NAME_LEN, \
             s_title(&average[parti].groups));
      maxgroup=s_size(&average[parti].groups);

      for (group=0; group < maxgroup; group++) {
         if (s_value(&average[parti].groups, group)) {
            dispchar = nextsymbol(group, ((0 == parti) && (0 == group)));
            if ((0 != parti) && (s_value(&average[parti - 1].groups, group))) {
               if (NULL != (pos = strchr(previous, dispchar))) {
                  display[pos - previous] = dispchar;
	       }
	    } else {
               pos = nextpos(display, previous, maxgroup);
               if (NULL != pos) {
                  *pos = dispchar;
	       } else {
                  swerr(proc, "Can't display mean groups", 0);
                  return(FALSE);
	       } 
	    } 
         }
      } /* for group */

      fprintf(outfile, "%.*s\n", (int)maxgroup, display);
      inc_linecount();
      strcpy(previous, display);
      sprintf(display,  "%*s", (int)maxgroup, " ");
   } /* for parti */

   PrintLine(outfile, linechar, screenwidth, "\n");
   inc_linecount();
#ifdef EXTRA_GROUPS
   if (showall && (3 <= howmany)) {
      NewLine(1, stdout);
      puts("Groups of 3 or more means can be further broken down.");
      inc_linecount();
      puts("See Sachs ([12] in the reference list) for further details.");
      inc_linecount();
      NewLine(1, stdout);
   }
#endif
   return(TRUE);
} /* show_means() */


char symbol(const elem_t id) {
   static const char symbollist[]  = "ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
                                     "abcdefghijklmnopqrstuvwxyz" \
                                     "!@#$%&*+-";
   static const unsigned maxsymbol = sizeof(symbollist);

   if ( (0 <= id) && (id < maxsymbol) ) {
     return(symbollist[id]);
   } else {
      return('?');
   }
}/* symbol() */


char * nextpos(      char   curr[],
                     char   prev[],
               const elem_t maxlen)
{
   char * pos = NULL;
   elem_t offset;

   if  ((NULL != (pos = strchr(curr, ' ')) && (pos - curr < maxlen))) {
         /* Try to find a better spot for the group symbol */
         /*   try to put it in a column with a blank in the */
         /*   previous line */
         offset = pos - curr;
         while ((offset < maxlen) && \
                ((' ' != curr[offset]) || (' ' != prev[offset]))) {
               offset++;
         }
         if (offset < maxlen) {
            pos = &(curr[offset]);
	 }
   }
   return (pos);
} /* nextpos() */


char nextsymbol(const elem_t group,
                const bool   first_time)
{
   static elem_t max_group_so_far;
   static char   symbol_list[MAX_PARTICIPANT + 1];

   if (first_time) {
      max_group_so_far = 0;
      sprintf(symbol_list, "%*s", (int)MAX_PARTICIPANT, " ");
   }
   if (' ' == symbol_list[group]) { 
      symbol_list[group] = symbol(max_group_so_far++);
   }
   return(symbol_list[group]);
} /* nextsymbol() */


/*                                                                         *\
   -----------------------------------------------------------------------
                       Files to Display (all output)
   -----------------------------------------------------------------------
\*                                                                         */

static const string TitleStr[] = {"No such item",
                                  "Data file format",
                                  "Data file example",
                                  "extract.sh script",
                                  "Complete listing of extract.awk",
                                  "References",
                                  "Author information",
                                  "Warranty Disclaimer for f2c",
                                  "Program overview"};

static const string ErrorDisp[]  = 
  {"Error!", 
   NULL};

static const string FormatDisp[] = 
  {"   The input file is a text file composed of groups of 251 lines.",
   "Each of these groups represents the data for one participant and ",
   "must have the following format:",
   "          line #  data (* = blank separated)",
   "            1     participant name",
   "         /  2     query name",
   "         |  3*    interpolated recall at fixed precision",
   "once for |         (0,.1,.2,.3,.4,.5,.6,.7,.8,.9,1)",
   "each of /   4     average precision (non-interpolated) over all",
   "the  50 \\          relevant documents",
   "queries. |  5*    precision at document retrieval level",
   "         |           (5,10,15,20,30,100,200,500,1000)",
   "         \\  6     precision after num_rel [not read in] documents "
                           "retrieved",
   "",
   "   The two names are strings of characters, all the other data are ",
   "real numbers.  The extract.sh Bourne shell script will convert from",
   "TREC datafiles to a single input file.  It uses the extract.awk AWK",
   "program to do most of the work.",
   "",
   "   N.B.: it is assumed that the query names are the same for every",
   "participant.  The names given for the first participant's queries ",
   "are the only ones used.",
   "",
   NULL};

static const string ExampleDisp[] =
  {"   Here are the first 11 lines on an actual input file and the",
   "first 64 lines of the original TREC datafile from which it was",
   "created.",
   "",
   "---Datafile begins---",
   "ACQNT1",
   "151",
   "1.0000 0.7500 0.6296 0.4828 0.2742 0.1210 0.0701 0.0000 0.0000 "
        "0.0000 0.0000",
   "0.2830",
   "1.0000 0.8000 0.6667 0.6500 0.6000 0.3300 0.1850 0.0960 0.0520",
   "0.3529",
   "152",
   "0.2857 0.0885 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000 0.0000 "
        "0.0000 0.0000",
   "0.0168",
   "0.0000 0.2000 0.1333 0.1000 0.1667 0.1600 0.1200 0.0960 0.0760",
   "0.0911",
   "---End of datafile extract---",
   ""
   "---Beginning of TREC mailfile---",
   "",
   "Queryid (Num):       151  ACQNT1",
   "Total number of documents over all queries",
   "    Retrieved:     1000",
   "    Relevant:        85",
   "    Rel_ret:         52",
   "Interpolated Recall - Precision Averages:",
   "    at 0.00       1.0000 ",
   "    at 0.10       0.7500 ",
   "    at 0.20       0.6296 ",
   "    at 0.30       0.4828 ",
   "    at 0.40       0.2742 ",
   "    at 0.50       0.1210 ",
   "    at 0.60       0.0701 ",
   "    at 0.70       0.0000 ",
   "    at 0.80       0.0000 ",
   "    at 0.90       0.0000 ",
   "    at 1.00       0.0000 ",
   "Average precision (non-interpolated) over all rel docs",
   "                  0.2830 ",
   "Precision:",
   "  At    5 docs:   1.0000",
   "  At   10 docs:   0.8000",
   "  At   15 docs:   0.6667",
   "  At   20 docs:   0.6500",
   "  At   30 docs:   0.6000",
   "  At  100 docs:   0.3300",
   "  At  200 docs:   0.1850",
   "  At  500 docs:   0.0960",
   "  At 1000 docs:   0.0520",
   "R-Precision (precision after R (= num_rel for a query) docs "
        "retrieved):",
   "    Exact:        0.3529",
   "",
   "Queryid (Num):       152  ACQNT1",
   "Total number of documents over all queries",
   "    Retrieved:     1000",
   "    Relevant:       538",
   "    Rel_ret:         76",
   "Interpolated Recall - Precision Averages:",
   "    at 0.00       0.2857 ",
   "    at 0.10       0.0885 ",
   "    at 0.20       0.0000 ",
   "    at 0.30       0.0000 ",
   "    at 0.40       0.0000 ",
   "    at 0.50       0.0000 ",
   "    at 0.60       0.0000 ",
   "    at 0.70       0.0000 ",
   "    at 0.80       0.0000 ",
   "    at 0.90       0.0000 ",
   "    at 1.00       0.0000 ",
   "Average precision (non-interpolated) over all rel docs",
   "                  0.0168 ",
   "Precision:",
   "  At    5 docs:   0.0000",
   "  At   10 docs:   0.2000",
   "  At   15 docs:   0.1333",
   "  At   20 docs:   0.1000",
   "  At   30 docs:   0.1667",
   "  At  100 docs:   0.1600",
   "  At  200 docs:   0.1200",
   "  At  500 docs:   0.0960",
   "  At 1000 docs:   0.0760",
   "R-Precision (precision after R (= num_rel for a query) docs "
        "retrieved):",
   "    Exact:        0.0911",
   "",
   "---End of TREC mailfile extract---",
   NULL};

static const string BourneDisp[] = 
  {"   The extract.sh script (below) takes as parameters the names of ",
   "the files of raw data provided by TREC in their standard format and",
   "sends to the standard output device a datafile that can be used by ",
   "the program",
   "",
   "",
   "Example:",
   "  ./extract.sh mailfile.PARTIAAA mailfile.PARTIBBB > IR-SP.input",
   "",
   "",
   "Complete listing of extract.sh:",
   "#! /bin/sh",
   "",
   "PATH=/usr/bsd43/bin:/usr/ucb:/bin:/usr/bin; export PATH",
   "EXTRACT=./extract.awk",
   "",
   "PROG=`basename $0`;",
   "USAGE=\"USAGE: ${PROG} filename(s)",
   "         where each filename is the name of a text file",
   "         in the format of a TREC adhoc \\`mailfile' file\"",
   "verbose=\"on\"",
   "",
   "if [ $# -lt 1 ]; then",
   "   echo $USAGE ; exit ",
   "fi",
   "",
   "while [ \"$1\" ]; do",
   "   SRC=$1;",
   "   LEN=`wc -l $SRC|awk '{print $1}'`",
   "   if [ $LEN -ne 1685 ]; then",
   "     echo \"file \\`$SRC' is the wrong length ($LEN "
                  "lines) -- skipping \" >&2",
   "   else",
   "     if [ \"$verbose\" ]; then",
   "        echo \"processing file $SRC\" >&2",
   "     fi",
   "     head -1600 $SRC | nawk -f extract.awk",
   "   fi",
   "   shift",
   "done",
   NULL};

static const string AWKDisp[] =
   /* 
     The script could be faster if not_first_time were used instead of f_t.
     The value would automatically be initialized to 0 
   */
  {"#! /bin/nawk",
   "    BEGIN { first_time = 1 }",
   "",
   "    { ",
   "       if (first_time) { ",
   "          LINE = NR ",
   "          if (2 == LINE) { print $4 }   "
        "# participant name",
   "       } else {",
   "          LINE = (NR % 32)",
   "       }",
   "",
   "       if ( 2 == LINE)                   { print $3 }"
        "          # query number",
   "       if (( 8 <= LINE) && (18 >= LINE)) "
        "{ printf(\"%s \", $3) } # recall at fixed",
   "       if (20 == LINE)                   "
        "{ print \"\\n\" $1 }     # average prec",
   "       if ((22 <= LINE) && (30 >= LINE)) "
        "{ printf(\"%s \", $4) } # prec at numdoc",
   "       if ((0 == LINE) || (32 == LINE))  "
                  "{ print \"\\n\" $2 }     # R-precision",
   "",
   "       if ((first_time) && (32 == NR))   "
                  "{ first_time = 0 }",
   "     }",
   NULL};

static const string ReferDisp[] = 
  {"I.  Other programs used by this program",
   "    -----------------------------------",
   " 1] Jgraph version 8.3 obtained using FTP from princeton.edu in",
   "   directory pub with filename jgraph.Z",
   "    Jgraph, created by Jim Plank <jsp@princeton.edu>, is used to produce",
   "   Postscript(tm) figures.  Note: Postscript is a registered trademark of",
   "   Adobe Systems Incorporated.", 
   "",
   " 2] Ghostscript and Ghostview",
   "    2a] Ghostscript",
   "      was originally created, and is primarily developed and maintained, ",
   "      by L. Peter Deutsch, President of Aladdin Enterprises.  It is ",
   "      copyright (C) 1989, 1992, 1993 Aladdin Enterprises.  Ghostscript ",
   "      related mail should be sent to <ghost@aladdin.com>.",
   "    2b] Ghostview",
   "      was written by Tim Theisen and is copyright (C) 1992 Timothy O. ",
   "      Theisen.  Ghostview related mail should be sent to ",
   "      <ghostview@cs.wisc.edu>.",
   "  Both can both be obtained using anonymous FTP from prep.ai.mit.edu in ",
   "  the pub/gnu directory.  The GNU Public licence applies to both.",
   "",
   "II. Published Algorithms and Code used in this program",
   "    --------------------------------------------------",
   "    N.B.: For complete details about changes, corrections and updates",
   "          see the source files.",
   "",
   " 3] F_{max} Table for Hartley test from ",
   "     _Biometrika Tables for Statisticians_ (Volume 1, 3rd edition) ",
   "     by E. S. Pearson and H. O. Hartley (eds.), 1966 [Table 31]",
   "     as reprinted in ",
   "        _Intermediate Statistical Methods and Applications: A ",
   "        Computer Package Approach_ by Mark L. Berenson, David M. ",
   "        Levine and Matthew Goldstein (c)1983 by Prentice-Hall, ",
   "        [Table B.5, page 549]",
   "",
   " 4] Code for ",
   "    4a] critical value of chi-squared ",
   "    4b] critical value of F",
   "    4c] normal z value",
   "    obtained using anonymous FTP from",
   "    netlib.att.com in directory netlib/a with filename perlman.Z",
   "    Used with permission of the author, Garry Perlman ",
   "    <perlman@cis.ohio-state.edu>.",
   "",
   "    He notes that the algorithm to compute the probability ",
   "    of the chi square value was adapted from ",
   "      Hill, I. D. and Pike, M. C.  ",
   "      Collected Algorithms for the CACM 1967 p. 243 (Algorithm 299)",
   "    and updated for rounding errors based on remark in ",
   "      ACM TOMS June 1985, page 185",
   "    and that the algorithm to compute the probability of the F ",
   "    ratio is adapted from",
   "      Egon Dorrer",
   "      Collected Algorithms of the CACM (Algorithm 322)",
   "    and that the algorithm to compute the probability of normal z ",
   "    value is adapted from a polynomial approximation in:",
   "      Ibbetson D, Algorithm 209",
   "      Collected Algorithms of the CACM 1963 p. 616",
   "",
   " 5] The Box test for equality of variances is used when there are more",
   "  than 12 samples.  The test is taken from ",
   "    Supplement 4A `A Robust Test of the Equality of Several Variances'",
   "    (pp. 167-8) of _Intermediate Business Statistics: Analysis of",
   "    Variance, Regression, and Time Series_ by Robert B. Miller and Dean",
   "    W. Wichern; Published by Holt, Rinehart and Winston ((c) 1977)",
   "    in the Series in Quantitative Method for Decision Making.",
   "  They cite the following source for the test:",
   "    `Robust Large-Sample Tests for Homogeneity of Variances' by M. W.",
   "    Layard in Journal of the American Statistical Association (March ",
   "    1973, Vol. 68, No. 341, pp. 195-8).",
   "",
   " 6] The Q value (studentized range) is computed using translations",
   "   of Fortran programs to C.  The translation was done using f2c [7]. ",
   "  The algorithms are",
   "    6a] ALGORITHM AS 190 APPL. STATIST. (1983) VOL.32, No.2",
   "      obtained from http://lib.stat.cmu.edu/apstat/190.",
   "      AS190 is from Applied Statistics v.32 no.2 (pp. 204 -- 210) ",
   "      with corrections from v.34 no.1 (p.104)",
   "    6b] ALGORITHM AS 111 APPL. STATIST., VOL.26, 118-121, 1977.",
   "      obtained from http://lib.stat.cmu.edu/apstat/111",
   "      AS 111 is from Applied Statistics (1977) v.26, pp. 118-121.",
   "    6c] Algorithm AS66 Applied Statistics (1973) vol22 no.3",
   "      obtained from http://lib.stat.cmu.edu/apstat/66",
   "      Algorithm AS66 is from Applied Statistics (1973) v.22 no.3.",
   "",
   " 7] The f2c program is copyright 1990 - 1995 by AT&T Bell Laboratories ",
   "   and Bellcore.  Select option 10 in the menu to see its accompanying",
   "   warranty disclaimer.",
   "     f2c was obtained by anonymous FTP from netlib.att.com in the ",
   "   netlib/f2c directory.",
   "",
   "III. Suggested Readings", 
   "     ------------------",
   "10] Mark L. Berenson, David M. Levine and Matthew Goldstein.",
   "   _Intermediate Statistical Methods and Applications: a computer ",
   "   package approach_, 1983: Prentice-Hall",
   "",   
   "11] Donna Harman. Overview of the first TREC conference.",
   "   In SIGIR '93 Proceedings, pp. 36-47, 1993.",
#ifdef EXTRA_GROUPS
   "",   
   "12] Lothar Sachs. _Angewandte Statistik_ in translation as: ",
   "   _Applied statistics: a handbook of techniques_ (second edition)",
   "   Translated by Zenon Reynarowych and copyright 1984 by Springer-Verlag.",
   "   Part of the Springer series in statistics.",
   "   (The section about detecting homogenous groups of means is on pages",
   "   512 - 514 and 533 - 537.)",
   "",
   "13] Rudolf J. Freund and William J. Wilson",
   "   _Statistical Methods_",
   "   Copyright 1993 by Academic Press, Inc., ",
   "   Published by Harcourt Brace Jovanovich",
   "   (Pages 236-247 contain a survey of methods for comparing groups of ",
   "   means.)",
#endif
   NULL};

static const string AuthorDisp[] =
  {"This program was written by James Blustein <jamie@csd.uwo.ca> using a", 
   "design prepared by Jean Tague-Sutcliffe.  The project was funded by ",
   "the American National Institute of Standards and Technology (NIST).",
   "",
   "Copyright (c) 1995 James Blustein",
   NULL};
  
static const string f2cDisclaimer[] = 
  {"f2c ([7] in the references list) was used to produce some of the  ",
   "code in this program.  AT&T requires that the notice below appear ",
   "in this supporting documentation and a comment in the source code.",
   "------------------------------------------------------------------",
   "f2c is Copyright 1990 - 1995 by AT&T Bell Laboratories and Bellcore.",
   "",
   "Permission to use, copy, modify, and distribute the f2c software",
   "and its documentation for any purpose and without fee is hereby",
   "granted, provided that the above copyright notice appear in all",
   "copies and that both that the copyright notice and this",
   "permission notice and warranty disclaimer appear in supporting",
   "documentation, and that the names of AT&T Bell Laboratories or",
   "Bellcore or any of their entities not be used in advertising or",
   "publicity pertaining to distribution of the software without",
   "specific, written prior permission.",
   "",
   "AT&T and Bellcore disclaim all warranties with regard to the f2c",
   "software, including all implied warranties of merchantability",
   "and fitness.  In no event shall AT&T or Bellcore be liable for",
   "any special, indirect or consequential damages or any damages",
   "whatsoever resulting from loss of use, data or profits, whether",
   "in an action of contract, negligence or other tortious action,",
   "arising out of or in connection with the use or performance of",
   "the f2c software.",
   NULL};

static const string ProgOverview[] = 
  {"   IR-STAT-PAK is a menu-driven suite of statistical tests for analysing",
   "and comparing the performance of a variety of queries over a variety of ",
   "of IR systems.  To make a selection from a menu: type the corresponding ",
   "number then press the Enter key.",
   "   Using data you choose from the entire data set the program can be    ",
   "used to:",
   "      * create Recall-Precision plots and histograms of precision data; ",
   "      * perform two- and three-way ANOVAs;",
   "      * create tables.",
   "   The program is designed to work with TREC result files but can be used",
   "with any dataset generated by standard IR tests as in trec.  TREC data  ",
   "consists of results of trials of many systems (called participants) on  ",
   "the same 50 queries.",
   "",
   "",
   "1 Detailed Instructions",
   "-----------------------",
   "   The program can only work with data that it reads in from a file in   ",
   "its own format. That format is explained in the file menu. The extract.sh",
   "and extract.awk files can be used together to create files in IR-STAT-PAK",
   "format from TREC mailfiles. An example of their use is in the file menu. ",
   "Once you have data files in the correct format, follow these             ",
   "instructions:                                                            ",
   "",
   "   1.   Read a datafile into the program. A file can be read from the ",
   "     command line when the program begins and at anytime from the file",
   "     menu.",
   "",
   "   2.   Select the participants and queries to include in the analysis or",
   "     display you intend to create. Initially, all participants and all  ",
   "     queries are selected.                                              ",
   "        The selections you make in menu 3 move participants and queries ",
   "     in an out of the list of selected data. Only the data in the       ",
   "     selected list will be used for the analysis or display. The last   ",
   "     item on the menu will allow you to select from the main menu.      ",
   "",
   "   3.   Select the type of data to analyse or display.  You must choose ",
   "     either single level or multilevel measure. You may select one or   ",
   "     both of the two-way measures: Average precision or R-precision (see",
   "     note). You may choose only one multilevel measure: precision at    ",
   "     fixed recall level or precision at fixed number of documents       ",
   "     retrieved.  To select the value of the measure to include, select  ",
   "     its corresponding menu item.  Selected single level measures appear",
   "     in the menu preceded by X. The selected levels of multilevel       ",
   "     measures appear on the menu immediately beneath the measure's name.",
   "     The last item on every menu returns you to the previous menu.      ",
   "        Note: R-precision is the precision after the number of documents",
   "     relevant to the query have been retrieved.                         ",
   "",
   "   4.   If you want to create plots or tables, select descriptive       ",
   "     statistics from the main menu. You can display tables of precision ",
   "     data or create Postscript figures for display, on an X Window      ",
   "     system, or print.                                                  ",
   "        If you have selected multilevel measures (see 3, above) then    ",
   "     Recall-Precision curves will be plotted for all selected           ",
   "     participants at all levels of interpolated recall.  If you have    ",
   "     selected single level measures, then the graph option will produce ",
   "     histograms.                                                        ",
   "        If you elect to have tables or histograms produced, you should  ",
   "     first select the level of averaging to use.                        ",
   "",
   "   5.   When you are finished using the program, select the last option ",
   "     from the main menu to quit.                                        ",
   "",
   "",
   "",
   "2 How it works",
   "--------------",
   "   There are four main categories of menus:",
   "     1. statistical analysis (menus 1 and 2);",
   "     2. data selection (menus 3 and 4);",
   "     3. data files (menu 5);",
   "     4. further options and documentation (menus 6 and 7).",
   "   Before any analysis or printing can take place, data must be read into",
   "the program. You can specify a filename to read on the command line or  ",
   "from the file menu.  When the program begins, it reads the data file ",
   "specified on the command line. If no filename is specified then data must",
   "be read from the file menu before any processing can occur. Once data has",
   "been read in, it can be analysed or used to print tables and graphs.",
   "   Before you select the analysis to perform (or tables and graphs to",
   "create) you will need to select a subset of the data with which to work.",
   "By default: the average precision for all participants and all queries is",
   "selected for all tasks.  Changes can be made in the data selection menus.",
   "If you do not make any selections then the entire data set will be     ",
   "selected. Changes to the participant and query selections are made from",
   "menu 3. Changes to the type of data (average precision, R-precision,   ",
   "interpolated precision at fixed recall level, precision at number of   ",
   "retrieved documents) can be made from menu 4.                          ",
   "   Once the data have been read in and selected, the program will be   ",
   "ready to perform analysis or create tables and graphs. The choices are ",
   "available from the first two menus.                                    ",
   "",
   "",
   "",
   "3  Some Details",
   "---------------",
   "3.1   The ANOVAs",
   "",
   "   If the data selected for analysis is multi-point (e.g., precision ",
   "measured at fixed recall levels) then you will have the choice of    ",
   "analysing it as by a series of two-way ANOVAs or one three-way ANOVA.",
   "Otherwise, a two-way ANOVA is computed. If the variances differ and  ",
   "the data has not been previously transformed, then the selected      ",
   "transform is applied. Currently, only the arcsine and logit transforms",
   "are available. Other such transforms can easily be included in the   ",
   "program code.  The ANOVAs produce tables showing the critical values ",
   "and a display of means in groups that do not differ by more than     ",
   "Tukey's honestly significant difference (see Berenson, et al [8] for ", 
   "details).", 
   "",
   "3.2   File format",
   "   To use the program you must supply a file of data in the correct   ",
   "format.  You can display a specification of that format from the data ",
   "files menu.  You can also use the accompanying extract.sh and extract.awk",
   "scripts to convert TREC data files into the IR-STAT-PAK format.  Once ",
   "you have a file in the correct format, it can be read into the program",
   "via the command line or file menu.",
   "",
   "",
   "3.3   What options are available",
   "   The options menu allows you to change the way the program works.  You",
   "can use it to:                                                          ",
   "      * change the transform to be applied;                             ",
   "      * set the condition for the transform to be applied;              ",
   "      * select verbose output;                                          ",
   "      * change the scrolling behaviour of the output;                   ",
   "      * change the maximum width of tables.                             ",
   "   By default, the selected transform is applied to change the data if, ",
   "during an ANOVA calculation, the variances of the participant data are  ",
   "found to differ significantly.  You may choose not to have a transform  ",
   "applied or to have it applied before any analysis occurs. With the last ",
   "option, you can print tables and create figures using transformed data. ",
   "The program will never transform data that has already been transformed.",
   "   By default, only the most important results of the program are       ",
   "displayed.  From the options menus you can select to have much of the   ",
   "intermediate results displayed during computation.  If you select to have",
   "intermediate results displayed then you can see what tests and critical ",
   "values the program is using.                                            ",
   "   The program pauses after printing a screen full of text (except in   ",
   "menus).  If your screen is larger or smaller than the default 24 lines  ",
   "then you should enter the correct figure from the options menu.  If you ",
   "are capturing output to a file you may want to turn off this behaviour  ",
   "-- set the screen height to 0 lines.                                    ",
   "",
   "   The program tries to display tables of descriptive statistics within ",
   "the width of your display.  By default, tables are wrapped at column 80.",
   "You may set a different size from the options menu. To see the default  ",
   "sizes of hard coded constants, such as the maximum table width, select  ",
   "`display program limits' from the `About the program' menu.             ",
   "",
   "",
   "3.4   The Program Code",
   "   The program is written in ANSI C.  The ABOUT and HOW-TO files in the",
   "distribution package document describe how various parts of the program",
   "work.  ABOUT describes the data structures in the program.  HOW-TO     ",
   "provides suggestions for how to make a number of changes to the        ",
   "program, e.g. to include a new transform.                              ",
   NULL};


static const string * Display[] = {ErrorDisp,
                                   FormatDisp,
                                   ExampleDisp,
                                   BourneDisp,
                                   AWKDisp,
                                   ReferDisp,
                                   AuthorDisp,
                                   f2cDisclaimer,
                                   ProgOverview};

void display_file(show which_file)
{
   extern const char         product[];  /* from main.c */
   extern const char         version[];  /* from main.c */
                ScreenSize   width;
   register     unsigned int line;

   start_menu();
   init_print_count();
   width = printf("%s version %s  %s", product, version, TitleStr[which_file]);
   NewLine(1, stdout);
   PrintLine(stdout, linechar, width, "");
   NewLine(1, stdout);
   for(line=0; Display[which_file][line]; line++) {
      puts(Display[which_file][line]);
      inc_linecount();
   }
   end_display();
} /* display_file() */


void limits(void) {
   extern const char product[];  /* from main.c */
   extern const char version[];  /* from main.c */

   start_menu();
   init_print_count();
   printf("%s version %s program limits information", product, version);
   NewLine(2, stdout);
   printf("  This version allows a maximum of %d participants.",
          MAX_PARTICIPANT);
   NewLine(1, stdout); 
   printf("Each participant name may be as long as %d characters.",
          PARTI_NAME_LEN);
   NewLine(1, stdout); 
   printf("Each of the %d queries per participant may have a name with",
          NUMofQUERIES);
   NewLine(1, stdout); 
   printf("as many as %d characters.", QUERY_NAME_LEN);
   NewLine(1, stdout); 
   printf("  Maximum supported display width (for displaying lists of");
   NewLine(1, stdout); 
   printf("participants and queries) is "SSZ_SPEC", current value is "\
          SSZ_SPEC ".", MAXscreenwidth, width_display());
   NewLine(1, stdout); 

   printf("Maximum supported (finite) display length is "SSZ_SPEC".", \
          MAXscreenlines);
   NewLine(1, stdout); 

   printf("  Filenames can be as long as %d characters.", (MAXFNAMELEN-1));
   NewLine(1, stdout); 
   printf("---------------------------------------------------------------");
   NewLine(1, stdout); 
   end_display();
 } /* limits() */

/*                                                                         *\
   -----------------------------------------------------------------------

   -----------------------------------------------------------------------
\*                                                                         */

